package com.zanxiang.game.data.serve.service.impl;

import com.zanxiang.game.data.serve.pojo.dto.PromotionDayDTO;
import com.zanxiang.game.data.serve.pojo.dto.PromotionDayTotalDTO;
import com.zanxiang.game.data.serve.pojo.vo.PromotionDayTotalVO;
import com.zanxiang.game.data.serve.pojo.vo.PromotionDayVO;
import com.zanxiang.game.data.serve.service.IAdsPromotionDayService;
import com.zanxiang.game.data.serve.utils.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.nutz.dao.Cnd;
import org.nutz.dao.Dao;
import org.nutz.dao.Sqls;
import org.nutz.dao.pager.Pager;
import org.nutz.dao.sql.Criteria;
import org.nutz.dao.sql.Sql;
import org.nutz.dao.util.Daos;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.List;

/**
 * @author tianhua
 * @time 2023/7/27
 * @Description
 **/
@Service
@Slf4j
public class PromotionDayServiceImpl implements IAdsPromotionDayService {

    @Autowired
    private Dao dao;

    @Override
    public Page<PromotionDayVO> getPromotionDayData(PromotionDayDTO dto) {
        //不传递查询条件默认查询当天数据
        if (dto.getCostBeginDate() == null || dto.getCostEndDate() == null) {
            dto.setCostBeginDate(LocalDate.now());
            dto.setCostEndDate(LocalDate.now());
        }
        //创建查询条件
        Criteria cri = Cnd.cri();
        if (dto.getAccountId() != null) {
            //拼接推广账号Id查询条件
            cri.where().andEquals("account_id", dto.getAccountId());
        }
        if (StringUtils.isNotBlank(dto.getAccountName())) {
            //拼接推广账号名称查询条件
            cri.where().andEquals("account_name", dto.getAccountName());
        }
        if (StringUtils.isNotBlank(dto.getAccountType())) {
            //推广账号类型
            cri.where().andEquals("account_type", dto.getAccountType());
        }
        if (dto.getPitcherId() != null) {
            cri.where().andEquals("pitcher_id", dto.getPitcherId());
        }
        if (StringUtils.isNotBlank(dto.getPitcherName())) {
            cri.where().andEquals("pitcher_name", dto.getPitcherName());
        }
        if (dto.getAgentId() != null) {
            cri.where().andEquals("agent_id", dto.getAgentId());
        }
        if (StringUtils.isNotBlank(dto.getAgentKey())) {
            cri.where().andEquals("agent_key", dto.getAgentKey());
        }
        if (StringUtils.isNotBlank(dto.getAgentName())) {
            cri.where().andEquals("agent_name", dto.getAgentName());
        }
        if (StringUtils.isNotBlank(dto.getCpName())) {
            cri.where().andEquals("cp_name", dto.getCpName());
        }
        if (StringUtils.isNotBlank(dto.getGameName())) {
            cri.where().andEquals("game_name", dto.getGameName());
        }
        if (dto.getGameId() != null) {
            cri.where().andEquals("game_id", dto.getGameId());
        }
        if (dto.getClassify() != null) {
            cri.where().andEquals("classify", dto.getClassify());
        }
        if (StringUtils.isNotBlank(dto.getPromotionName())) {
            cri.where().andEquals("promotion_name", dto.getPromotionName());
        }
        if (dto.getPromotionId() != null) {
            cri.where().andEquals("promotion_id", dto.getPromotionId());
        }
        if (StringUtils.isNotBlank(dto.getStatus())) {
            cri.where().andEquals("status", dto.getStatus());
        }
        if (StringUtils.isNotBlank(dto.getProjectName())) {
            cri.where().andEquals("project_name", dto.getProjectName());
        }
        if (dto.getProjectId() != null) {
            cri.where().andEquals("project_id", dto.getProjectId());
        }
        if (dto.getCostBeginDate() != null && dto.getCostEndDate() != null) {
            cri.where().andBetween("dt", dto.getCostBeginDate(), dto.getCostEndDate());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            cri.where().andEquals("source_system", dto.getSourceSystem());
        }
        //拼接分组条件
        cri.getGroupBy().groupBy("promotion_id");
        //创建sql语句
        Sql sql = Sqls.create(getPromotionDayDataSql() + cri );
        //添加自定义回传对象
        sql.setCallback(Sqls.callback.entities());
        sql.setEntity(dao.getEntity(PromotionDayVO.class));
        //设置pager对象
        Pager pager = dao.createPager(dto.getPageNum(), dto.getPageSize());
        sql.setPager(pager);
        //执行sql
        dao.execute(sql);
        //获取到结果list
        List<PromotionDayVO> list = sql.getList(PromotionDayVO.class);
        //分页对象设置总条数
        Sql sqlCount = Sqls.queryEntity("select count(*) from ads_promotion_day" + cri);
        pager.setRecordCount((int) Daos.queryCount(dao, sqlCount));

        //返回查询得结果
        return new Page<>(list,pager);
    }

    /**
     * 查询广告监控数据
     * @return
     */
    private String getPromotionDayDataSql() {
        return """
                SELECT
                	promotion_id,
                	MAX(promotion_name) as promotion_name,
                	MAX(project_id) as project_id,
                	MAX(project_name) as project_name,
                	MAX(account_name) as account_name,
                	MAX(account_id) as account_id,
                	MAX(account_type) as account_type,
                	MAX(pitcher_id) as pitcher_id,
                	MAX(pitcher_name) as pitcher_name,
                	MAX(agent_id) as agent_id,
                	MAX(agent_name) as agent_name,
                	MAX(agent_key) as agent_key,
                	MAX(cp_name) as cp_name,
                	MAX(game_id) as game_id,
                	MAX(game_name) as game_name,
                	MAX(classify) as classify,
                	SUM(today_cost) as today_cost,
                	SUM(show_count) as show_count,
                	round(if(SUM(show_count) > 0, SUM(today_cost) / SUM(show_count), 0), 2) as thousand_display_price,
                	SUM(click_count) as click_count,
                	round(if(SUM(click_count) > 0, SUM(today_cost) / SUM(click_count), 0), 2) as avg_click_cost,
                	round(if(SUM(show_count) > 0, SUM(click_count) / SUM(show_count), 0), 4) as ctr,
                	SUM(convert_count) as convert_count,
                	SUM(convert_cost) as convert_cost,
                	round(if(SUM(click_count) > 0, SUM(convert_count) / SUM(click_count), 0), 4) as convert_rate,
                	SUM(reg_num) as reg_num,
                	SUM(first_role_num) as first_role_num,
                	SUM(new_user_total_role_num) as new_user_total_role_num,
                	round(if(SUM(reg_num) > 0, SUM(today_cost) / SUM(reg_num), 0), 2) as reg_cost,
                	round(if(SUM(first_role_num) > 0, SUM(today_cost) / SUM(first_role_num), 0), 2) as first_role_cost,
                	round(if(SUM(new_user_total_role_num) > 0, SUM(today_cost) / SUM(new_user_total_role_num), 0), 2) as new_user_total_role_cost,
                	round(if(SUM(reg_num) > 0, SUM(first_role_num) / SUM(reg_num), 0), 4) as first_role_rate,
                	round(if(SUM(reg_num) > 0, SUM(new_user_total_role_num) / SUM(reg_num), 0), 4) as new_user_total_role_rate,
                	SUM(first_new_user_amount_count) as first_new_user_amount_count,
                	SUM(first_new_user_amount_num) as first_new_user_amount_num,
                	SUM(first_new_user_amount) as first_new_user_amount,
                	SUM(new_user_total_amount_count) as new_user_total_amount_count,
                	SUM(new_user_total_amount_num) as new_user_total_amount_num,
                	SUM(new_user_total_amount) as new_user_total_amount,
                	round(if(SUM(today_cost) > 0, SUM(first_new_user_amount) / SUM(today_cost), 0), 4) as first_roi,
                	SUM(twenty_four_hours_amount) as twenty_four_hours_amount,
                	round(if(SUM(today_cost) > 0, SUM(twenty_four_hours_amount) / SUM(today_cost), 0), 4) as twenty_four_hours_roi,
                	round(if(SUM(today_cost) > 0, SUM(new_user_total_amount) / SUM(today_cost), 0), 4) as total_roi,
                	round(if(SUM(first_new_user_amount_num) > 0, SUM(today_cost) / SUM(first_new_user_amount_num), 0), 2) as first_new_user_recharge_cost,
                	round(if(SUM(new_user_total_amount_num) > 0, SUM(today_cost) / SUM(new_user_total_amount_num), 0), 2) as new_user_total_recharge_cost,
                	round(if(SUM(first_new_user_amount_num) > 0, SUM(first_new_user_amount) / SUM(first_new_user_amount_num), 0), 2) as first_new_user_arppu,
                	round(if(SUM(new_user_total_amount_num) > 0, SUM(new_user_total_amount) / SUM(new_user_total_amount_num), 0), 2) as new_user_total_amount_arppu,
                	SUM(first_new_user_hundred_user_num) as first_new_user_hundred_user_num,
                	round(if(SUM(first_new_user_hundred_user_num) > 0, SUM(today_cost) / SUM(first_new_user_hundred_user_num), 0), 2) as first_new_user_hundred_user_cost,
                	SUM(first_recharge_fifty_hundred_num) as first_recharge_fifty_hundred_num,
                	round(if(SUM(new_user_total_amount_num) > 0, SUM(first_recharge_fifty_hundred_num) / SUM(new_user_total_amount_num), 0), 4) as first_recharge_fifty_hundred_rate,
                	SUM(first_new_user_two_hundred_user_num) as first_new_user_two_hundred_user_num,
                	round(if(SUM(first_new_user_two_hundred_user_num) > 0, SUM(today_cost) / SUM(first_new_user_two_hundred_user_num), 0), 2) as first_new_user_two_hundred_user_cost,
                	SUM(new_user_total_hundred_user_num) as new_user_total_hundred_user_num,
                	round(if(SUM(new_user_total_hundred_user_num) > 0, SUM(today_cost) / SUM(new_user_total_hundred_user_num), 0), 2) as new_user_total_hundred_user_cost,
                	SUM(first_ios_amount_num) as first_ios_amount_num,
                	SUM(first_ios_amount_count) as first_ios_amount_count,
                	SUM(first_ios_amount) as first_ios_amount,
                	round(if(SUM(first_new_user_amount_num) > 0, SUM(first_ios_amount_num) / SUM(first_new_user_amount_num), 0), 4) as first_ios_amount_num_rate,
                	round(if(SUM(first_new_user_amount) > 0, SUM(first_ios_amount) / SUM(first_new_user_amount), 0), 4) as first_ios_amount_rate,
                	round(if(SUM(today_cost)> 0, SUM(first_ios_amount) / SUM(today_cost), 0), 4) as first_ios_amount_roi,
                	SUM(first_android_amount_count) as first_android_amount_count,
                	SUM(first_android_amount_num) as first_android_amount_num,
                	SUM(first_android_amount) as first_android_amount,
                	round(if(SUM(first_new_user_amount_num) > 0,SUM(first_android_amount_num) / SUM(first_new_user_amount_num), 0), 4) as first_android_amount_num_rate,
                	round(if(SUM(first_new_user_amount) > 0, SUM(first_android_amount) / SUM(first_new_user_amount), 0), 4) as first_android_amount_rate,
                	round(if(SUM(today_cost) > 0, SUM(first_android_amount) / SUM(today_cost), 0), 4) as first_android_amount_roi,
                	round(if(SUM(reg_num) > 0, SUM(first_new_user_amount_num) / SUM(reg_num), 0), 4) as first_amount_rate,
                	round(if(SUM(first_new_user_amount_count) > 0, SUM(first_new_user_amount) / SUM(first_new_user_amount_count), 0), 2) as first_new_user_avg_price,
                	round(if(SUM(new_user_total_amount_count) > 0, SUM(new_user_total_amount) / SUM(new_user_total_amount_count), 0), 2) as new_user_total_avg_price,
                	SUM(d2) as d2,
                	SUM(d3) as d3,
                	SUM(d7) as d7,
                	SUM(d15) as d15
                FROM
                	game_ads.ads_promotion_day
                """;
    }

    @Override
    public PromotionDayTotalVO getPromotionDayTotalData(PromotionDayTotalDTO dto) {
        return null;
    }
}
